/**************************************************************

    This file is a part of the Empirical Likelihood library
 for Stata. This file contains a simple Continuous Updating
 GMM (CUE) estimator function.

 Written by Kirill Evdokimov and Yuichi Kitamura

 This version: 02/03/2011

***************************************************************/

version 10.0
mata:
mata set matastrict on

/*
   Function 'CUE' calculates Continuously Updating GMM estimator (CUE).
This function numerically minimizes the criterion function 'CUE_criterion'
over the parameter 'b'.

   Function arguments:

   mc_f        - [IN]  the moment conditions function
   b_cue       - [IN]  starting point
                 [OUT] CUE estimator value
   cue_val     - [OUT] the value of the CUE criterion function
   optimize_tracelevel - [IN] Instructs the 'optimize' procedure
                               how detailed the printouts are to be.

   Arguments mc_f and b_cue are required, all the other arguments are optional

   The function returns 0 when no errors were detected,
and a negative error code when something went wrong.

   This function uses Mata's optimization procedure 'optimize'.
The code uses BFGS optimization method. Other methods can be used;
see the help to function 'optimize'.
*/
real scalar CUE(pointer scalar mc_f, real colvector b_cue
  , | real scalar cue_val, string optimize_tracelevel)
{
  external pointer scalar cue_mc_f
  cue_mc_f = mc_f;

  external transmorphic optimization_S /*this is the main object that contains all information used by
    and produced by the '_optimize()' function. If optimization fails 'optimization_S' can be used
    to discover the reasons by checking 'optimize_result_errorcode(optimization_S)' and
    'optimize_result_errortext(optimization_S)'. */

   if (args()<4) { optimize_tracelevel = "step"; }
  if (args()<2) {
    b_cue = .; cue_val = .;
    errprintf("CUE: Wrong number of arguments.\n");
    return(-1);
  }

  optimization_S = optimize_init();
  optimize_init_which(optimization_S,"min");
  optimize_init_technique(optimization_S, "bfgs");
  optimize_init_valueid(optimization_S,"CUE(b)");
  optimize_init_tracelevel(optimization_S, optimize_tracelevel);
  optimize_init_evaluator(optimization_S, &CUE_optimize_eval());
  optimize_init_params(optimization_S, b_cue'); //starting value

  //stopping criteria settings
  optimize_init_conv_ptol(optimization_S,1e-5); //tolerances; you may want to modify these
  optimize_init_conv_vtol(optimization_S,1e-5);
  optimize_init_conv_maxiter(optimization_S,10000); //maximum numer of iterations

  real scalar rc
  rc = _optimize(optimization_S);
  if (!rc) {
    b_cue = optimize_result_params(optimization_S)';
    cue_val = optimize_result_value(optimization_S);
  } else { /*Optimization failed. The error code can be optained by running
    'optimize_result_errorcode(optimization_S)' and the meaning can be found
    in Mata manual for "optimize()" function or by calling 'optimize_result_errortext(optimization_S)'. */
    b_cue= .; cue_val = .;
    errprintf("CUE: Optimization routine failed.\nSee the external 'optimization_S' for details.\n");
    return(-3); //optimization error while minimizing the first step GMM criterion function
  }
  return(0); //no error
}

//CUE criterion function
real scalar function CUE_criterion(real colvector b) {
  real matrix w, w_mean, omega, i_omega, crit
  external pointer scalar cue_mc_f
  w = (*cue_mc_f)(b);
  w_mean = mean(w,1);
  //w = w:-w_mean;
  omega=cross(w,w);
  i_omega = invsym(omega/st_nobs());
  crit = st_nobs()*w_mean*i_omega*w_mean';
  return(crit);
}

//'evaluator' wrapper function needed for the 'optimize()' routine
void CUE_optimize_eval(todo, x, y, g, H) {
  y = CUE_criterion(x');
  pragma unused g; pragma unused H; //we do not calculate gradient or hessian
}


//store these functions (compiled) in the 'lib_EL' library
mata mlib add lib_EL CUE() CUE_criterion() CUE_optimize_eval()

end

