/**************************************************************

    This file is a part of the Empirical Likelihood library
 for Stata. This file contains the Empirical Likelihood (EL)
 estimator function.

 Written by Kirill Evdokimov and Yuichi Kitamura

 This version: 02/03/2011

***************************************************************/


version 10.0
mata:
mata set matastrict on

/*
   Function 'EL' is the main routine that calculates the
Empirical Likelihood estimator. This function numerically
minimizes the "outerloop" over the parameter 'b'.

Function arguments:

   mc_f - [IN] moment conditions function

   b    - [IN]  starting point
          [OUT] estimator value

   fval - [OUT] the criterion value if successful

   optimize_tracelevel  - [IN] Instructs the 'optimize' procedure
                                how detailed the printouts are to be.
                                This argument is optional.

   The function returns 0 when no errors were detected,
and a negative error code when something went wrong.

   This function uses Mata's optimization procedure 'optimize'.
The code uses BFGS optimization method. Other methods can be used;
see the help to function 'optimize'. The code also explains
how to use Nelder-Mead optimization procedure instead.
*/
real scalar EL(pointer scalar mc_f, real colvector b, real scalar fval
               , | string optimize_tracelevel)
{
  external pointer scalar el_mc_f
  external real scalar el_innerloop_error_code, el_infinity
  real colvector b_error
  real scalar rc
  external colvector el_best_b
  external scalar el_best_val // 'el_best_val' and 'el_best_b' always contain the best value achived
                              // in the outerloop and the corresponding parameter value.
                              // These can be used if optimization fails.

  el_infinity = 1e50; //this constant is used by different EL and EL_innerloop functions

  if ((args()<2)||(args()>4)) {
    errprintf("EL: Invalid number of arguments\n");
    return(-1);
  }
  if (args()<4) { optimize_tracelevel = "step"; }

  b_error = J(rows(b),1,0); //return b=0 in case of an error
  el_mc_f = mc_f;
  fval = EL_innerloop( (*el_mc_f)(b) );
  if ((fval>=el_infinity) || (el_innerloop_error_code!=0)) {
    fval = -1;
    b = b_error;
    el_best_b = b_error; el_best_val = -1;
    errprintf("EL: Innerloop diverges at the starting value. You need to use another starting value.\n");
    return(-2);
  }

  el_best_b = b; el_best_val = fval; //best so far
  fval = 0;

  /* Now we start minimization over the parameter */

  external transmorphic el_optimization_S   /*this is the main object that contains all information used by
    and produced by the '_optimize()' function. If optimization fails 'el_optimization_S' can be used
    to find out the reasons by checking 'optimize_result_errorcode(el_optimization_S)' and
    'optimize_result_errortext(el_optimization_S)'. */


  el_optimization_S = optimize_init();
  optimize_init_which(el_optimization_S,"min");
  optimize_init_technique(el_optimization_S, "bfgs");

  //stopping criteria settings
  optimize_init_conv_ptol(el_optimization_S,1e-5); //tolerances; you may want to modify these
  optimize_init_conv_vtol(el_optimization_S,1e-5);
  optimize_init_conv_maxiter(el_optimization_S,10000); //maximum numer of iterations


/* Uncomment what is below if you are using Nelder Mead */
/* // Nelder-Mead initial simplex calculation
  real matrix simplex
  real rowvector nm_deltas
  real scalar p, i, min_simplex_side, initial_simplex_side_mul

  initial_simplex_side_mul = 0.01; //These two parameters determine initial simplex size;
  min_simplex_side = 0.001;        //they depend on the range of values of the parameter 'b'.

  p = length(b);
  nm_deltas = J(1, p, 0);

  for (i=1; i<=p; i++){
    nm_deltas[i]=(b[i])*initial_simplex_side_mul;
    if (abs(nm_deltas[i])<min_simplex_side)
      nm_deltas[i]=min_simplex_side;
  }
  optimize_init_nmsimplexdeltas(el_optimization_S,nm_deltas);
  optimize_init_technique(el_optimization_S, "nr");

// end of Nelder-Mead initial simplex calculation */

  optimize_init_valueid(el_optimization_S,"EL:");
  optimize_init_tracelevel(el_optimization_S, optimize_tracelevel);
  optimize_init_evaluator(el_optimization_S, &EL_outerloop_eval());
  optimize_init_evaluatortype(el_optimization_S, "d1"); //our evaluator
                        // can calculate both the  value and its gradient
  optimize_init_params(el_optimization_S, b'); //starting value

  rc = _optimize(el_optimization_S)';
  if (!rc) {
    b = optimize_result_params(el_optimization_S)';
    fval = optimize_result_value(el_optimization_S);
    return(0); //no errors
  } else {/*Optimization failed. The error code can be optained by running
    'optimize_result_errorcode(el_optimization_S)' and the meaning can be found
    in Mata manual for "optimize()" function or by calling 'optimize_result_errortext(el_optimization_S)'.

	We return the parameter b from the optimization procedure even if it failed to converge.
    At the same time, the value of the criterion function is set to missing.*/
    fval = .;
    errprintf("EL: Optimization routine failed.\nSee the external 'el_optimization_S' for details.\n");
    return(-3);
  }
}


/*

Function 'EL_outerloop_eval' is the standard evaluator function
that is required by Mata's 'optimize' procedure. This is a
wrapper for the 'EL_innerloop' function. This is a 'd1' type evaluator,
which means that it is able to calculate both innerloop value 'val' and
its gradient 'grad' at any point 'b'. Parameter 'todo' tells whether
we should only calculate the value of the function, or both the value
and the gradient. Parameter 'H' is not used.

*/
void function EL_outerloop_eval(todo, b, val, grad, H){
  real matrix w
  external pointer scalar el_mc_f
  external real scalar el_infinity, el_innerloop_error_code
  external real colvector el_lambda, el_probability
  real scalar el_use_two_sided_moment_gradient, i, eps
  real colvector mc_at_param, mc_at_plus, mc_at_minus, param
  external colvector el_best_b
  external scalar el_best_value // 'el_best_value' and 'el_best_b' always contain the best value achived
                              // in the outerloop and the corresponding parameter value.
                              // These can be used if optimization fails.


  mc_at_param = (*el_mc_f)(b'); //moment conditions at 'b'. Note that the moment conditions
                      // function takes a colvector as a parameter, while 'b' is a rowvector in Stata
  val = EL_innerloop(mc_at_param);   //obtain the value of innerloop at 'b'


  if ((val>=el_infinity) || (el_innerloop_error_code!=0)) {

    val=.; //innerloop diverged at 'b'. When the function (innerloop)
         //cannot be calculated at 'b', mf_optimize allows us to return '.'
         //and to not fill in 'grad'
    return;
  }

  if (el_best_value>val) {
    el_best_value = val;
    el_best_b = b'; //best so far
  }

  if (todo>=1) { //asked to obtain the gradient
    //The procedure below exploits nested structure of the optimization problem; it is
    //faster than numerical evaluation of derivative and should also be more precise.

    eps = 1e-6;
    el_use_two_sided_moment_gradient = 1; //Setting this nonzero will force "two-sided" calculation
          // of derivative (see the code below). This may improves precision, but is a bit slower.


    grad = J(1,cols(b),0); //note that 'grad' and 'b' are row vectors in Stata
    for (i=1; i<=cols(b); i++){
      param = J(cols(b),1,0);
      param[i]=1;
      mc_at_plus = (*el_mc_f)(b'+eps*param);
      if (el_use_two_sided_moment_gradient) {
        mc_at_minus = (*el_mc_f)(b'-eps*param);
        grad[1,i] = 2*el_probability'(mc_at_plus-mc_at_minus)/(2*eps)*el_lambda;
      }
      else {
        grad[1,i] = 2*el_probability'(mc_at_plus-mc_at_param)/eps*el_lambda;
      }
    }
  }
}


/*

Function 'EL_innerloop' calculates the value of the "innerloop".

Calculation of this function involves a simple numerical optimization.
The function returns the (nonegative) EL value if converged. Negative return
indicates that an error occured and the EL value was not obtained. One
important case when this happens is when the zero vector is not in the
convex hall of the moment conditions matrix 'w'. See 'description.txt'
for details.

Input argument:
  w    - <observations> x <number of moments> matrix of sample moment restrictions

Output:
  returns (nonnegative) EL value or (negative) error code if diverged

The scalar parameters el_max_innerloop_iterations, el_tolerance_grad_direc,
el_infinity, and el_innerloop_printdiag can be modified from outside of this function
affecting its behavior.

*/
real scalar function EL_innerloop(real matrix w) {
  real scalar n, numofmoments, lik, iter, test, testmax
  external real colvector el_lambda, el_probability //vector of implied EL probability weights
  real scalar el_max_innerloop_iterations, el_tolerance_grad_direc, el_min_step_mul, el_innerloop_printdiag
  external real scalar el_infinity, el_innerloop_error_code

  /* Setting up parameters */
  el_max_innerloop_iterations = 100;
  el_tolerance_grad_direc = 1e-7;
  el_min_step_mul = 1e-3;
  el_innerloop_printdiag = 0; //if nonzero error messages are printed


  el_innerloop_error_code = 0; //initialize error code
  n = rows(w);
  numofmoments = cols(w);
  el_lambda = J(numofmoments,1,0);
  el_probability = J(n,1,0);
  testmax = .999999*n; //if innerloop of EL diverges a criterion (see below) approaches 'n'

  /* Innerloop Optimization */

  iter = 0;
  while (++iter <= el_max_innerloop_iterations ) {
    el_probability = 1:/(1:+w*el_lambda);
    real matrix grad, direc;
    grad = el_probability:*w;
    direc = grad'grad;
    grad = -colsum(grad)';

    if (all(abs(direc):<el_infinity)==1 && all(abs(grad):<el_infinity)==1) {
      direc = invsym(direc)*grad;
      test = grad'direc;
    } else
      test = testmax + 1; //convergence criterion too large

    if (test > testmax) {
        if (el_innerloop_printdiag)
            errprintf("EL innerloop: diverged (after %f iterations)\n", iter);
        el_innerloop_error_code = 310;
        return(el_infinity);
    }

    lik = sum(ln(el_probability));
    if (grad'direc <= el_tolerance_grad_direc){
        return(-2*lik);
    }

    //looking for new lambda
    real scalar step, improved;
    real colvector lamtry, el_p;
    step = 1;
    improved = 0;

    while (step >= el_min_step_mul && !improved)  {
      lamtry = el_lambda :- step*direc;
      el_p = 1 :+ (w*lamtry);
      if (any(el_p :<= 0) == 0 && sum(-ln(el_p)) < lik) {
        //checked positivity and the criterion got smaller
        improved = 1;
        el_lambda = lamtry;
        el_probability = 1:/el_p;
      } else
        step = step/2;
    }

    if (!improved) {
        if (el_innerloop_printdiag)
            errprintf("EL innerloop: stepsize failed to find increase on iteration %f\n", iter);
        el_innerloop_error_code = 320;
        return(el_infinity);
    }
  }

  if (el_innerloop_printdiag)
    errprintf("EL innerloop: maximum number (%f) of iterations reached\n", el_max_innerloop_iterations);

  el_innerloop_error_code = 330; //maximum number of iterations was reached
  return(el_infinity);
}

//store these functions (compiled) in the 'lib_EL' library
mata mlib add lib_EL EL() EL_outerloop_eval() EL_innerloop()

end

