/**************************************************************

    This file is a part of the Empirical Likelihood library
 for Stata. This file contains a simple one- and two-step
 Generalize Method of Moments GMM estimator function.

 Written by Kirill Evdokimov and Yuichi Kitamura

 This version: 02/03/2011

***************************************************************/

version 10.0
set more off

mata:
mata set matastrict on

/*
   Function 'GMM' calculates one and two step GMM estimators.
This function numerically minimizes the criterion function
'GMM_criterion' over the parameter 'b'.

   Function arguments :

   mc_f        - [IN]  the moment conditions function
   b_gmm1      - [IN]  starting point
                 [OUT] one step GMM (GMM1) estimator value
   gmm1_val    - [OUT] the value of the GMM1 criterion function
   b_gmm2      - [OUT] two step GMM (GMM2) estimator value
   gmm2_val    - [OUT] the value of the GMM2 criterion function
   optimize_tracelevel - [IN] Instructs the 'optimize' procedure
                               how detailed the printouts are to be.

   Arguments mc_f and b_gmm1 are required, all the other arguments are optional

   The function returns 0 when no errors were detected,
and a negative error code when something went wrong.

   This function uses Mata's optimization procedure 'optimize'.
The code uses BFGS optimization method. Other methods can be used;
see the help to function 'optimize'.
*/
real scalar GMM(pointer scalar mc_f, real colvector b_gmm1,
    | real scalar gmm1_val, real colvector b_gmm2, real scalar gmm2_val, string optimize_tracelevel)
{
  external pointer scalar gmm_mc_f //the pointer to the moment condition function
  gmm_mc_f = mc_f;

  external real matrix weighting_matrix //GMM weighting matrix
  weighting_matrix = 1; //we start with the identity weighting matrix

  external transmorphic optimization_S /*this is the main object that contains all information used by
    and produced by the '_optimize()' function. If optimization fails 'optimization_S' can be used
    to discover the reasons by checking 'optimize_result_errorcode(optimization_S)' and
    'optimize_result_errortext(optimization_S)'. */

  real scalar onestep, rc

  if (args()<=6) { optimize_tracelevel = "step"; }
  onestep = (args()<=3); //perform one step GMM if we do not need to return b_gmm2
  if (args()<2) {
    b_gmm1 = .; gmm1_val = .; b_gmm2 = .; gmm2_val = .;
    errprintf("GMM: Wrong number of arguments.\n");
    return(-1); //invalid number of arguments
  }

  optimization_S = optimize_init();
  optimize_init_which(optimization_S,"min");
  optimize_init_technique(optimization_S, "bfgs");
  optimize_init_valueid(optimization_S,"GMM1(b)");
  optimize_init_tracelevel(optimization_S, optimize_tracelevel);
  optimize_init_evaluator(optimization_S, &GMM_optimize_eval());
  optimize_init_params(optimization_S, b_gmm1'); //starting value

  //stopping criteria settings
  optimize_init_conv_ptol(optimization_S,1e-5); //tolerances; you may want to modify these
  optimize_init_conv_vtol(optimization_S,1e-5);
  optimize_init_conv_maxiter(optimization_S,10000); //maximum numer of iterations

  rc = _optimize(optimization_S);
  if (!rc) {
    b_gmm1 = optimize_result_params(optimization_S)';
    gmm1_val = optimize_result_value(optimization_S);
  } else { /*Optimization failed. The error code can be optained by running
    'optimize_result_errorcode(optimization_S)' and the meaning can be found
    in Mata manual for "optimize()" function or by calling 'optimize_result_errortext(optimization_S)'. */
    b_gmm1 = .; gmm1_val = .; b_gmm2 = .; gmm2_val = .;
	errprintf("GMM1: Optimization routine failed.\nSee the external 'optimization_S' for details.\n");
    return(-3); //optimization error while minimizing the first step GMM criterion function
  }

  if (!onestep){
    real matrix w, w_mean, omega
    w = (*mc_f)(b_gmm1);
    w_mean = mean(w,1);
    //w = w:-w_mean;
    omega=cross(w,w);
    weighting_matrix = invsym(omega/st_nobs());
    optimize_init_valueid(optimization_S,"GMM2(b)");
    optimize_init_params(optimization_S, b_gmm1');
    rc = _optimize(optimization_S);
    if (!rc) {
      b_gmm2 = optimize_result_params(optimization_S)';
      gmm2_val = optimize_result_value(optimization_S);
    } else { /*Optimization failed. The error code can be optained by running
      'optimize_result_errorcode(optimization_S)' and the meaning can be found
      in Mata manual for "optimize()" function or by calling 'optimize_result_errortext(optimization_S)'. */
      b_gmm2 = .;  gmm2_val = .;
      errprintf("GMM2: Optimization routine failed.\nSee the external 'optimization_S' for details.\n");
      return(-4); //optimization error while minimizing the second step GMM criterion function
    }
  }
  return(0); //no errors
}

//GMM criterion function
real scalar function GMM_criterion(real colvector b) {
  real matrix w, w_mean, crit
  external pointer scalar gmm_mc_f
  external real matrix weighting_matrix
  w = (*gmm_mc_f)(b);
  w_mean = mean(w,1);
  crit = rows(w)*w_mean*weighting_matrix*w_mean';
  return(crit);
}

//'evaluator' wrapper function needed for the 'optimize()' routine
void GMM_optimize_eval(todo, x, y, g, H) {
  y = GMM_criterion(x');
  pragma unused g; pragma unused H; //we do not calculate gradient or hessian
}

//store these functions (compiled) in the 'lib_EL' library
mata mlib add lib_EL GMM() GMM_criterion() GMM_optimize_eval()

end
