/************************************************************************
*                                                                       *
*      Example1.do - this file contains an example of use               *
*   of the Empirical Likelihood (EL) estimator.  This file is a         *
*   part of the Empirical Likelihood library for Stata.                 *
*                                                                       *
*      This example estimates a dynamic panel model of earnings.        *
*   The data are taken from Altonji and Segal (1996, JBES).             *
*   The moment conditions are DIFF and SYS moment conditions            *
*   from Blundell and Bond (1998,JOE). The example calculates           *
*   GMM, CUE, and EL estimators. This example is similar to             *
*   an example in Imbens (2002, JBES), although a slightly              *
*   different dataset is used.                                          *
*                                                                       *
*      The model is E_{i,t} = theta * E_{i,t-1} + eta_{i} + eps_{i,t},  *
*   where E_{i,t}=log(Earnings_{i,t}), 'theta' is the autoregressive    *
*   parameter of interest, eta_{i} is the fixed effect, eps_{i,t}       *
*   is the idiosincratic shock.                                         *
*                                                                       *
*      The data is loaded from the accompanying file                    *
*   'example1_data.txt, which is obtained from Altonji and              *
*   Segal (1996,JBES).                                                  *
*                                                                       *
*                                                                       *
*                                                                       *
*      Written by Kirill Evdokimov and Yuichi Kitamura                  *
*                                                                       *
*      This version: 02/03/2011                                         *
*                                                                       *
************************************************************************/




clear
set more off

quietly
mata:

mata clear
mata set matastrict on
mata set matafavor speed
mata set matalnum off //to increase the speed, for debugging switch it on



/*****************************************************************
*   First we specify the function that loads and prepares data,  *
*           and the moment condition function                    *
******************************************************************/

/* This function loads and prepares data for the moment condition function */

void function load_and_prepare_the_data() {
  real matrix means_v_y
  real scalar rc, i, j
  external real matrix v_y, v_dy // these will be used by the
  external real scalar maxT      // moment condition function 'mc_dp'

  printf("Loading and preparing the data\n");

  rc = _stata("insheet using example1_data.txt, clear");
  if (rc) {
    errprintf("Error loading the data!\n");
    exit();
  }
  if (st_nobs()!=1536){
    errprintf("Wrong number of observations! Wrong dataset?\n");
    exit();
  }
  maxT = 11;
  v_y = st_data(., (1..maxT));
  v_y = log(v_y); //log of earnings data
  means_v_y = mean(v_y); //demean the data
  v_y = v_y - J(st_nobs(),1,means_v_y);
  v_dy = J(st_nobs(),maxT,.);
  v_dy[.,(2..maxT)] = v_y[.,(2..maxT)]-v_y[.,(1..maxT-1)];

  printf("There are 1536 observation, 54 moment conditions, and one parameter\n");
} /* load_and_prepare_the_data() */


/* This is the moment condition function */
real matrix function mc_dp(real scalar theta) {
/* These are moment conditions for dynamic panel from Blundell and Bond (1998),
   that are used to estimate the autoregressive parameter. The intercept and
   time effects are not estimated since the data were demeaned in advance. */

  external real matrix v_y, v_dy //these variables were prepared by 'load_and_prepare_the_data()'
  external real scalar maxT
  real scalar numofmoments, mc_i, j, s
  real matrix mc, de, w

  numofmoments = (maxT-1)*(maxT-2)/2+(maxT-2);
  mc = J(st_nobs(),numofmoments,0); //moment conditions
  mc_i = 1; //moment condition index
  for (j=3;j<=maxT;j++){ //DIF moment conditions
    de = v_dy[.,j]-theta*v_dy[.,j-1];
    for (s=1;s<=j-2;s++){
      mc[.,mc_i]=v_y[.,s]:*de;
      mc_i++;
    }
  }
  for (j=2;j<=maxT;j++){  //SYS moment conditions
    w = v_y[.,j]-theta*v_y[.,j-1];
    if (j>=3){
      mc[.,mc_i]=v_dy[.,j-1]:*w;
      mc_i++;
    }
  }
  return(mc);
} /* end of the moment condition function mc_dp() */

/* define analogs of functions 'tic' and 'toc' in Matlab */
void function tic() { timer_clear(99); timer_on(99); }
real function toc() { timer_off(99); timer_on(99); return (timer_value(99)[1]); }

/* This function displays the results from all estimation methods together */
function display_all_results(real b_gmm1, real gmm1_val, real b_gmm2, real gmm2_val, real gmm_time
           , real b_cue, real cue_val, real cue_time, real b_el, real el_val, real el_time,
           | real ci_left, real ci_right)
{
  printf("{txt}\n{hline}\nAll results together:\n");
  printf("{txt}GMM1: theta = {res:%4.2f}, val = {res:%6.2f}\n", b_gmm1, gmm1_val);
  printf("{txt}GMM2: theta = {res:%4.2f}, val = {res:%6.2f}. The calculation of both steps took %3.1f sec\n"
           , b_gmm2, gmm2_val, gmm_time);
  printf("{txt}CUE : theta = {res:%4.2f}, val = {res:%6.2f}. The calculation took %3.1f sec\n", b_cue, cue_val, cue_time);
  printf("{txt}EL  : theta = {res:%4.2f}, val = {res:%6.2f}. The calculation took %3.1f sec\n", b_el, el_val, el_time);

  if (args()==13)
    printf("{txt}\n95%% EL confidence interval is {res:(%4.2f,%4.2f)}\n", ci_left, ci_right);
  printf("{hline}\n");
} /* end of display_all_results() */

/*****************************************************************
*              end of function declarations                      *
*****************************************************************/



/******************************************************/
/*         The main program starts here               */

load_and_prepare_the_data();

tic();
b_gmm1 = 0; //starting value
tic();
ret = GMM(&mc_dp(), b_gmm1, gmm1_val=., b_gmm2=., gmm2_val=., "tolerance");
gmm_time = toc();
printf("{txt}GMM1: theta = {res:%4.2f}, val = {res:%6.2f}\n", b_gmm1, gmm1_val);
printf("{txt}GMM2: theta = {res:%4.2f}, val = {res:%6.2f}\n", b_gmm2, gmm2_val);
printf("The calculation of GMM took %3.1f seconds\n", gmm_time);

b_cue = b_gmm2; //starting value for CUE is the GMM2 estimate
tic();
ret = CUE(&mc_dp(), b_cue, cue_val=.);
cue_time = toc();
printf("{txt}CUE : theta = {res:%4.2f}, val = {res:%6.2f}. The calculation took %3.1f seconds\n", b_cue, cue_val, cue_time);

b_el = b_gmm2; //starting value for EL is the GMM2 estimate
tic();
ret = EL(&mc_dp(), b_el, el_val=.);
el_time = toc();

display_all_results(b_gmm1, gmm1_val, b_gmm2, gmm2_val, gmm_time, b_cue, cue_val, cue_time, b_el, el_val, el_time);

//we now obtain confidence for the parameter by inverting the EL criterion function:
printf("{txt}\nObtaining confidence interval for the parameter by inverting the EL criterion function:\n");
grid =  0.3:+(0::40)/200;
vals = J(rows(grid),1,0);
for (i=1;i<=rows(grid); i++){
  printf("{txt}Point {res:%2.0f} out of {res:%2.0f}. Theta = {res:%5.3f}.", i, rows(grid), grid[i]);
  EL_outerloop_eval(1, grid[i], v=., g=., H=.); //g and H are not used
  if (!missing(v))
    vals[i] = v;
  else /* 'v' is missing when the innerloop diverges. The innerloop
         diverges when zero is not in the convex hull of the moment conditions. */
    vals[i] = -1;
  printf("{txt} EL criterion function = {res:%6.2f}\n", v);
}
crit_value = el_val+invchi2(1, 0.95);
ci_left = .;
ci_right = .;
for (i=1; i<=rows(grid); i++){
  if (vals[i]>0 & vals[i]<crit_value){
    ci_left = grid[i];
    break;
  }
}
for (i=rows(grid); i>=1; i--){
  if (vals[i]>0 & vals[i]<crit_value){
    ci_right = grid[i];
    break;
  }
}
/* Please note that in general EL confidence intervals may not have the
   simple interval form, and may consist of a union of intervals. */

/* The above calculation should only be seen as an illustration.
   For practical work you may want to explore finer grids around
   the boundaries of confidence interval, use interpolation,
   and/or use bisection/Newton root searching algorithms. */


display_all_results(b_gmm1, gmm1_val, b_gmm2, gmm2_val, gmm_time
   , b_cue, cue_val, cue_time, b_el, el_val, el_time, ci_left, ci_right);

end
