/****************************************************************************
*                                                                           *
*      Example2.m - this file contains an example of use                    *
*   of the Empirical Likelihood (EL) estimator.  This file is a             *
*   part of the Empirical Likelihood library for Stata.                     *
*                                                                           *
*      This example considers minimum distance estimation of the            *
*   covariance structure of earnings and hours, based on                    *
*   Abowd and Card (1989,Ecta). The model of this example is                *
*   described in Section 5 of Altonji and Segal (1996,JBES).                *
*   The sample consists of 1536 men, with data on changes in                *
*   (log-) earnings and hours for years 1969-1979. Assuming                 *
*   stationarity, there are 39 parameters to estimate                       *
*   (hours variance, 9 hours autocovariances, earnings variance,            *
*   9 earnings autocovariances, and 19 hours/earnings covariances).         *
*   There are 210 moment conditions, which are coded in the function        *
*   'mc_md' that can be found in the file 'example2.m'. We calculate        *
*   GMM, CUE, and EL estimators.                                            *
*                                                                           *
*      The data is loaded from the accompanying file 'example2_data.txt,    *
*   which is obtained from the data of Altonji and Segal (1996,JBES).       *
*                                                                           *
*                                                                           *
*      Written by Kirill Evdokimov and Yuichi Kitamura                      *
*                                                                           *
*      This version: 02/03/2011                                             *
*                                                                           *
****************************************************************************/



clear
set more off

mata:
mata clear
mata set matastrict on
mata set matalnum off //to increase the speed, for debugging switch it on
mata set matafavor speed



/*****************************************************************
*   First we specify the function that loads and prepares data,  *
*           and the moment condition function                    *
******************************************************************/


/*
  This function loads and prepares data for the moment condition function

  The data are loaded from the accompanying file 'example2_data.txt'.
*/

void function load_and_prepare_the_data() {
  real matrix v_h, v_e, v_h2, v_e2, v_hA, v_eA, v_heA
  real scalar rc, i, j
  external real matrix v_all /* this will be used
       by the moment condition function 'mc_md' */

  printf("Loading and preparing the data\n");
  rc = _stata("insheet using example2_data.txt, clear");
  if (rc) {
    errprintf("Error loading the data!\n");
    exit();
  }
  if (st_nobs()!=1536){
    errprintf("Wrong number of observations! Wrong dataset?\n");
    exit();
  }
  v_h = st_data(., (1..10));  //hours
  v_e = st_data(., (11..20)); //earnings

v_h2 = v_h:^2; //hours variance

  v_e2 = v_e:^2; //earnings variance
  /* now we calculate hours autocovariances and earnings autocovariances */

  v_hA = v_h[.,2]:*v_h[.,1];
  v_eA = v_e[.,2]:*v_e[.,1];
  for(i=3; i<=10; i++){

    for(j=1; j<=i-1; j++){
      v_hA = (v_hA, v_h[.,i]:*v_h[.,j]);
      v_eA = (v_eA, v_e[.,i]:*v_e[.,j]);
    }
  }
  /* now we calculate hours and earnings covariances */
  v_heA = J(st_nobs(),100,0);
  for (i=1; i<=10; i++){
    for (j=1; j<=10; j++){
      v_heA[.,(i-1)*10+j] = v_h[.,i]:*v_e[.,j];
    }
  }
  v_all = (v_h2, v_e2, v_hA, v_eA, v_heA); /* this will be used by
                                the moment condition function 'mc_md' */
  printf("We have 1536 observations, 210 moments, and 39 parameters\n");
}/* load_and_prepare_the_data() */


/* This is the moment condition function */
real matrix function mc_md(real colvector params) {
/* This is a set of moment conditions for PSID data as described in
   Altonji-Segal(1996)(originally Abowd and Card, 1987) */

/* 'params' vector includes 39 parameters:
   params(1) - hours variances
   params(2:10) - 9 hours autocovariances
   params(11) - 9 earnings variance
   params(12:20) - 9 earnings autocovariances
   params(21:39) - 19 hours/earnings covariances */

  if (length(params)!=39) {
    errprintf("Error: Incorrect number of parameters!\n");
    exit();
  }

  external real matrix v_all
  real matrix w
  real scalar index, i, j
  real colvector h_var, h_cov, e_var, e_cov, he_cov, unit_vec

  h_var = params[1];       // params(1) - hours variance
  h_cov = params[2..10];   // params(2:10) - 9 hours autocovariances
  e_var = params[11];      // params(11) - earnings variance
  e_cov = params[12..20];  // params(12:20) - 9 earnings autocovariances
  he_cov = params[21..39]; // params(21:39) - 19 hours/earnings autocovariances

  index = 1; //current moment condition number we are dealing with
  unit_vec = J(st_nobs(),1,1);
  w = v_all;
  // hours covariance
  for (i=1;i<=10;i++){
    w[.,index] = w[.,index]-h_var*unit_vec;
    index++;
  }
  // earnings covariance
  for (i=1;i<=10;i++){
    w[.,index] = w[.,index]-e_var*unit_vec;
    index++;
  }
  // hours autocovariances
  for (i=2;i<=10;i++){
    for (j=1;j<=i-1;j++){
      w[.,index] = w[.,index]-h_cov[i-j]*unit_vec;
      index++;
    }
  }
  // earnings autocovariances
  for (i=2;i<=10;i++){
    for (j=1;j<=i-1;j++){
      w[.,index] = w[.,index]-e_cov[i-j]*unit_vec;
      index++;
    }
  }
  // hours/earnings covariances
  // eh_cov(t) = Cov(HOURS(j),EARNINGS(j+10+t)) where t can run from -9 to 9
  for (i=1;i<=10;i++){
    for (j=1;j<=10;j++){
      w[.,index] = w[.,index]-he_cov[10+i-j]*unit_vec;
      index++;
    }
  }
  return(w);
} /* end of the moment condition function mc_md() */


/* This function displays the estimated parameters in a convenient way */
void function display_results(title, b,| val, time_took){
  printf("{txt}\n{hline}\n%s\n", title);
  printf("{txt}      LAG:           T=0     T=1     T=2\n");
  printf("{txt}Cov(E(t),E(t-T))   {res:%6.3f}  {res:%6.3f}  {res:%6.3f}\n",b[11],b[12],b[13]);
  printf("{txt}Cov(H(t),H(t-T))   {res:%6.3f}  {res:%6.3f}  {res:%6.3f}\n",b[1], b[2], b[3]);
  printf("{txt}Cov(H(t),E(t-T))   {res:%6.3f}  {res:%6.3f}  {res:%6.3f}\n",b[30],b[31],b[32]);
  printf("{txt}Cov(E(t),H(t-T))   {res:%6.3f}  {res:%6.3f}  {res:%6.3f}\n",b[30],b[29],b[28]);
  if (args()>=3) {
    printf("{txt}Over-id test: {res:%3.1f}\n", val);
  }
  if (args()==4) {
    printf("The calculation took %3.1f seconds\n", time_took);
  }
} /* end of 'display_results' function */

/* define analogs of functions 'tic' and 'toc' in Matlab */
void function tic() { timer_clear(99); timer_on(99); }
real function toc() { timer_off(99); timer_on(99); return (timer_value(99)[1]); }

/*            end of function declarations                 */
/***********************************************************/



/***********************************************************/
/*             The main program starts here                */

load_and_prepare_the_data();
b_gmm1 = J(39,1,0); //starting value
tic();
ret = GMM(&mc_md(), b_gmm1, gmm1_val=., b_gmm2=., gmm2_val=.);
gmm_time = toc();
display_results("  GMM1 Results:", b_gmm1, gmm1_val);
display_results("  GMM2 Results:", b_gmm2, gmm2_val);
printf("The calculation of GMM took %3.1f seconds\n", gmm_time);

b_cue = b_gmm2; //starting value for CUE is the GMM1 estimate
tic();
ret = CUE(&mc_md(), b_cue, cue_val=.);
cue_time = toc();
display_results(" CUE Results:", b_cue, cue_val, cue_time);

b_el = b_cue; //starting value for EL is the CUE estimate
tic();
ret = EL(&mc_md(), b_el, el_val=.);
el_time = toc();
display_results("  EL Results:", b_el, el_val, el_time);


printf("\n\n{hline}\n\n  All results together:\n");
printf("\n  Here:\n   'E' stands for changes in log-Earnings,\n   'H' stands for changes in log-Hours,\n   and T=0,1,2 is the lag of the covariance.\n\n");

display_results("  GMM1 Results:", b_gmm1, gmm1_val);
display_results("  GMM2 Results:", b_gmm2, gmm2_val, gmm_time);
display_results("  CUE Results:", b_cue, cue_val, cue_time);
display_results("  EL Results:", b_el, el_val, el_time);

/*               End of the main program                   */
/***********************************************************/

end


